pub enum Message {
Show 19 variants
Version(VersionMessage),
Verack,
Ping(Nonce),
Pong(Nonce),
Reject {
message: String,
ccode: RejectReason,
reason: String,
data: Option<[u8; 32]>,
},
GetAddr,
Addr(Vec<MetaAddr>),
GetBlocks {
known_blocks: Vec<Hash>,
stop: Option<Hash>,
},
Inv(Vec<InventoryHash>),
GetHeaders {
known_blocks: Vec<Hash>,
stop: Option<Hash>,
},
Headers(Vec<CountedHeader>),
GetData(Vec<InventoryHash>),
Block(Arc<Block>),
Tx(UnminedTx),
NotFound(Vec<InventoryHash>),
Mempool,
FilterLoad {
filter: Filter,
hash_functions_count: u32,
tweak: Tweak,
flags: u8,
},
FilterAdd {
data: Vec<u8>,
},
FilterClear,
}Expand description
A Bitcoin-like network message for the Zcash protocol.
The Zcash network protocol is mostly inherited from Bitcoin, and a list of Bitcoin network messages can be found on the Bitcoin wiki.
That page describes the wire format of the messages, while this enum stores an internal representation. The internal representation is unlinked from the wire format, and the translation between the two happens only during serialization and deserialization. For instance, Bitcoin identifies messages by a 12-byte ascii command string; we consider this a serialization detail and use the enum discriminant instead. (As a side benefit, this also means that we have a clearly-defined validation boundary for network messages during serialization).
Variants
Version(VersionMessage)
A version message.
Note that although this is called version in Bitcoin, its role is really
analogous to a ClientHello message in TLS, used to begin a handshake, and
is distinct from a simple version number.
Verack
A verack message.
Ping(Nonce)
A ping message.
Tuple Fields
0: NonceA nonce unique to this Self::Ping message.
Pong(Nonce)
A pong message.
Tuple Fields
0: NonceThe nonce from the Self::Ping message this was in response to.
Reject
A reject message.
Fields
ccode: RejectReasonRejectReason code relating to rejected message.
GetAddr
A getaddr message.
Addr(Vec<MetaAddr>)
A sent or received addr message, or a received addrv2 message.
Currently, Zebra:
- sends and receives
addrmessages, - parses received
addrv2messages, ignoring some address types, - but does not send
addrv2messages.
The list contains 0..=MAX_META_ADDR addresses.
Because some address types are ignored, the deserialized vector can be empty, even if the peer sent addresses. This is not an error.
GetBlocks
A getblocks message.
known_blocks is a series of known block hashes spaced out along the
peer’s best chain. The remote peer uses them to compute the intersection
of its best chain and determine the blocks following the intersection
point.
The peer responds with an inv packet with the hashes of subsequent blocks.
If supplied, the stop parameter specifies the last header to request.
Otherwise, an inv packet with the maximum number (500) are sent.
The known blocks list contains zero or more block hashes.
Fields
Inv(Vec<InventoryHash>)
An inv message.
Allows a node to advertise its knowledge of one or more
objects. It can be received unsolicited, or in reply to
getblocks.
The list contains zero or more inventory hashes.
GetHeaders
A getheaders message.
known_blocks is a series of known block hashes spaced out along the
peer’s best chain. The remote peer uses them to compute the intersection
of its best chain and determine the blocks following the intersection
point.
The peer responds with an headers packet with the headers of subsequent blocks.
If supplied, the stop parameter specifies the last header to request.
Otherwise, the maximum number of block headers (160) are sent.
The known blocks list contains zero or more block hashes.
Fields
Headers(Vec<CountedHeader>)
A headers message.
Returns block headers in response to a getheaders packet.
Each block header is accompanied by a transaction count.
The list contains zero or more headers.
GetData(Vec<InventoryHash>)
A getdata message.
getdata is used in response to inv, to retrieve the
content of a specific object, and is usually sent after
receiving an inv packet, after filtering known elements.
zcashd returns requested items in a single batch of messages.
Missing blocks are silently skipped. Missing transaction hashes are
included in a single notfound message following the transactions.
Other item or non-item messages can come before or after the batch.
The list contains zero or more inventory hashes.
Block(Arc<Block>)
A block message.
Tx(UnminedTx)
A tx message.
This message can be used to:
- send unmined transactions in response to
GetDatarequests, and - advertise unmined transactions for the mempool.
Zebra chooses to advertise new transactions using Inv(hash) rather than Tx(transaction).
NotFound(Vec<InventoryHash>)
A notfound message.
Zebra responds with this message when it doesn’t have the requested blocks or transactions.
When a peer requests a list of transaction hashes, zcashd returns:
- a batch of messages containing found transactions, then
- a
notfoundmessage containing a list of transaction hashes that aren’t available in its mempool or state.
But when a peer requests blocks or headers, any missing items are
silently skipped, without any notfound messages.
The list contains zero or more inventory hashes.
Mempool
FilterLoad
A filterload message.
This was defined in BIP37, which is included in Zcash.
Zebra currently ignores this message.
Fields
filter: FilterThe filter itself is simply a bit field of arbitrary byte-aligned size. The maximum size is 36,000 bytes.
FilterAdd
A filteradd message.
This was defined in BIP37, which is included in Zcash.
Zebra currently ignores this message.
FilterClear
A filterclear message.
This was defined in BIP37, which is included in Zcash.
Zebra currently ignores this message.
Implementations
Sourceimpl Message
impl Message
Sourcepub fn inv_strategy() -> BoxedStrategy<Self>
pub fn inv_strategy() -> BoxedStrategy<Self>
Create a strategy that only generates Message::Inv messages.
Sourcepub fn get_data_strategy() -> BoxedStrategy<Self>
pub fn get_data_strategy() -> BoxedStrategy<Self>
Create a strategy that only generates Message::GetData messages.